unit Layout;

interface

uses Windows, NUtil, Element, Gauge;

type
  TLayouter = class(TObject)
    X, Width, Height: Integer;
    R, G, B: Integer;
    BgR, BgG, BgB, BgA: Integer;
    FontName: string;
    FontSize: Integer;
    FontBold: Boolean;
    FontItalic: Boolean;
    Elements: TObjList;
    constructor Create;
    destructor Destroy; override;
    procedure Draw(DC: HDC);
    procedure Update;
    procedure Eval(const S: string; const Gauges: TGauges);
    procedure Load(const Path: string; const Gauges: TGauges);
    function NameToClass(const Name: string): TElementClass;
  end;

implementation

uses NLib, NConf, LayoutDefault;

constructor TLayouter.Create;
begin
  Elements := TObjList.Create;
end;

destructor TLayouter.Destroy;
begin
  Elements.Free;
  inherited;
end;

procedure TLayouter.Draw(DC: HDC);
var
  i: Integer;
  font, oldfont: HFONT;
  fn: string;
  fs: Integer;
  fb: Boolean;
  fi: Boolean;
  fw: DWORD;
begin
  fn := '';
  fs := 0;
  fb := False;
  fi := False;
  font := 0;
  oldfont := 0;
  SetBkMode(DC, TRANSPARENT);
  for i := 0 to Elements.Count-1 do
    with Elements[i] as TElement do
    begin
      if (FontName <> fn) or (FontSize <> fs) or (FontBold <> fb) or
        (FontItalic <> fi) then
      begin
        if FontBold then fw := FW_BOLD else fw := FW_NORMAL;
        if (font <> 0) and (oldfont <> 0) then
        begin
          SelectObject(DC, oldfont);
          DeleteObject(font);
        end;
        font := CreateFont(FontSize, 0, 0, 0, fw, Ord(FontItalic), 0, 0,
          DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS,
          DEFAULT_QUALITY, DEFAULT_PITCH, PChar(FontName));
        oldfont := SelectObject(DC, font);
      end;
      Draw(DC);
    end;
  SelectObject(DC, oldfont);
  DeleteObject(font);
end;

procedure TLayouter.Update;
var
  i: Integer;
begin
  for i := 0 to Elements.Count-1 do
    (Elements[i] as TElement).Update;
end;

function TLayouter.NameToClass(const Name: string): TElementClass;
var
  i: Integer;
begin
  for i := 0 to High(ElementClasses) do
    if ElementClasses[i].ClassNameIs('T'+Name) then
    begin
      Result := ElementClasses[i];
      Exit;
    end;
  Result := nil;
end;

procedure TLayouter.Eval(const S: string; const Gauges: TGauges);
var
  ac: Integer;
  av: array[0..1023] of string;
  c: TElementClass;
begin
  SplitCmd(S, ac, av);      
  if av[0] = 'width' then
    Width := SInt(av[1])
  else if av[0] = 'bgcolor' then
  begin
    BgR := SInt(av[1]);
    BgG := SInt(av[2]);
    BgB := SInt(av[3]);
  end
  else if av[0] = 'bgalpha' then
    BgA := SInt(av[1])
  else if av[0] = 'space' then
    Inc(Height, SInt(av[1]))
  else if av[0] = 'color' then
  begin
    R := SInt(av[1]);
    G := SInt(av[2]);
    B := SInt(av[3]);
  end
  else if av[0] = 'font' then
  begin
    FontName := av[1];
    FontSize := SInt(av[2]);
    FontBold := (av[3] = 'bold') or (av[4] = 'bold');
    FontItalic := (av[3] = 'italic') or (av[4] = 'italic');
  end
  else begin
    c := NameToClass(av[0]);
    if Assigned(c) then
    begin
      Elements.Add(c.Create(X, Height, Width-X, SInt(av[1]), r, g, b,
        FontName, FontSize, FontBold, FontItalic, ac, av, Gauges));
      Inc(Height, SInt(av[1]));
    end;
  end;
end;

procedure TLayouter.Load(const Path: string; const Gauges: TGauges);
var
  i: Integer;
begin
  Elements.Clear;
  // Default attributes
  X := 1; // left margin
  Width := 48;
  Height := 0;
  R := 255;
  G := 255;
  B := 255;
  BgA := 128;
  FontName := 'Arial';
  FontSize := 14;
  FontBold := False;
  FontItalic := False;
  try
    with TConf.Create(Path) do
      try
        while ReadLine do
          Eval(Line, Gauges);
      finally
        Free;
      end;
  except
    on E: EIOError do // use default layout
      for i := 0 to High(LayoutDefault.Layout) do
        Eval(LayoutDefault.Layout[i], Gauges);
  end;
end;

end.
